#include <iostream>
#include <vector>
#include <algorithm>
#include <functional>

template<typename Container, class Compare>
Container sort(const Container& arg, Compare cmp)
{
    auto result = arg;
    std::sort(result.begin(), result.end(), cmp);
    return result;
}


template<typename T>
using Cmp = std::function<bool(const T&, const T&)>;

struct almost_equal_to
{
    template<typename T>
    bool operator()(const T& lhs, const T& rhs, int ulp) const
    {
        static_assert(std::is_floating_point_v<T>, "T must be floating point type");

        return std::abs(lhs - rhs) <= std::numeric_limits<T>::epsilon() * std::abs(lhs + rhs) * ulp
            || std::abs(lhs - rhs) < std::numeric_limits<T>::min();
    }
};

struct equal_to
{
    template<typename T>
    bool operator()(const T& lhs, const T& rhs) const
    {
        if constexpr (std::is_floating_point_v<T>)
        {
            return almost_equal_to()(lhs, rhs, 2);
        }
        else
        {
            return lhs == rhs;
        }
    }
};

template<typename T>
bool equal_to_fun(const T& lhs, const T& rhs)
{
    if constexpr (std::is_floating_point_v<T>)
    {
        return almost_equal_to()(lhs, rhs, 2);
    }
    else
    {
        return lhs == rhs;
    }

};


int main(void)
{
    std::vector<Cmp<double>> comparators;

    comparators.push_back(equal_to_fun<double>);
    comparators.push_back(equal_to());
    comparators.push_back([](const double& a, const double& b)->bool {return a < b; });

    std::vector<double> v0{ 0.1, 0.2, 1.0/std::sqrt(5.0)/std::sqrt(5.0), 0.3 };

    for (auto & comparator : comparators)
    {
        v0 = sort(v0, comparator);
    }

    return 0;
}
