#include "ranking.hpp"
#include "meeting.hpp"

std::ostream& operator<<(std::ostream& os, const RankingRecord& record)
{
    os << record.team_ << " : " << record.wins_ << "," << record.losts_ << "," << record.ties_ << "," << record.points_;
    return os;
}

bool operator<(const RankingRecord& lhs, const RankingRecord& rhs)
{
    return lhs.points_ < rhs.points_;
}

std::ostream& operator<<(std::ostream& os, const Ranking& ranking)
{
    for (auto& ranking_record : ranking.ranking_)
        os << ranking_record.second << "\n";
    return os;
}

std::tuple<
    SimplePointsCalculator::Points, 
    SimplePointsCalculator::Win, 
    SimplePointsCalculator::Lost, 
    SimplePointsCalculator::Tie
> SimplePointsCalculator::calculate(const Meeting& meeting, const Team& team)
{
    if (!meeting.played)
    {
        return NOT_PLAYED;
    }

    if (meeting.hosting_result == meeting.visiting_result)
    {
        return TIE;
    }

    if (meeting.hosting_result > meeting.visiting_result && meeting.hosting == team ||
        meeting.hosting_result < meeting.visiting_result && meeting.visiting == team)
    {
        return WIN;
    }

    return LOST;
}

RankingRecord::RankingRecord(const Team& team)
    : team_(team)
{
}

void RankingRecord::add(const Meeting& meeting)
{
    auto [points, win, lost, tie] = SimplePointsCalculator().calculate(meeting, team_);

    points_ += points;
    wins_ += win;
    losts_ += lost;
    ties_ += tie;
}

Ranking::Ranking(const Meetings& meetings)
{
    for (auto& meeting : meetings)
    {
        add(meeting);
    }
}

void Ranking::add(const Meeting& meeting)
{
    add(meeting.hosting);
    add(meeting.visiting);

    ranking_.find(meeting.hosting)->second.add(meeting);
    ranking_.find(meeting.visiting)->second.add(meeting);
}

void Ranking::add(const Team& team)
{
    if (ranking_.end() == ranking_.find(team))
    {
        ranking_.insert(std::make_pair(team, RankingRecord(team)));
    }
}
