#ifndef RANKING_HPP
#define RANKING_HPP

#include <string>
#include <vector>
#include <map>
#include <tuple>

#include "team.hpp"

struct Meeting;
class Meetings;

class SimplePointsCalculator
{
    using Win = bool;
    using Lost = bool;
    using Tie = bool;
    using Points = int;

    using Result = std::tuple<Points, Win, Lost, Tie>;

    constexpr static const Result WIN = Result(3, true, false, false);
    constexpr static const Result LOST = Result(0, false, true, false);
    constexpr static const Result TIE = Result(1, false, false, true);
    constexpr static const Result NOT_PLAYED = Result(0, false, false, false);
public:
    Result calculate(const Meeting&, const Team&);
};

class RankingRecord
{
    friend std::ostream& operator<<(std::ostream&, const RankingRecord&);
    friend bool operator< (const RankingRecord&, const RankingRecord&);

public:
    RankingRecord(const Team&);

    void add(const Meeting&);

private:
    Team team_;
    int points_ = 0;
    unsigned int wins_ = 0;
    unsigned int losts_ = 0;
    unsigned int ties_ = 0;
};

class Ranking
{
    friend std::ostream& operator<<(std::ostream&, const Ranking&);

public:
    Ranking(const Meetings&);

    void add(const Meeting&);

private:
    void add(const Team&);

    std::map<Team, RankingRecord> ranking_;
};

#endif // RANKING_HPP
